@@ -1,19 +1,22 @@
-#include <ATen/ATen.h>
-#include <ATen/cuda/CUDAContext.h>
 #include <ATen/Config.h>
+#include <ATen/core/DimVector.h>
+#include <ATen/cuda/CUDAContext.h>
 #include <ATen/native/cuda/CuFFTUtils.h>
 #include <ATen/native/utils/ParamsHash.h>
+#include <c10/util/accumulate.h>
+#include <c10/util/irange.h>
 
-#include <list>
-#include <unordered_map>
-#include <string>
-#include <stdexcept>
-#include <sstream>
-#include <limits>
 #include <cufft.h>
 #include <cufftXt.h>
 
-namespace at { namespace native { namespace detail {
+#include <limits>
+#include <list>
+#include <sstream>
+#include <stdexcept>
+#include <string>
+#include <unordered_map>
+
+namespace at::native::detail {
 
 // Enum representing the FFT type
 enum class CuFFTTransformType : int8_t {
@@ -55,7 +58,7 @@ struct CuFFTParams
   }
 };
 
-static_assert(std::is_trivial<CuFFTParams>::value, "");
+static_assert(std::is_trivial_v<CuFFTParams> );
 
 // Returns true if the transform type has complex input
 inline bool cufft_complex_input(CuFFTTransformType type) {
@@ -109,7 +112,7 @@ public:
 
   ~CuFFTHandle() {
 // Not using fftDestroy() for rocFFT to work around double freeing of handles
-#ifndef __HIP_PLATFORM_HCC__
+#if !defined(USE_ROCM)
     cufftDestroy(handle_);
 #endif
   }
@@ -120,11 +123,7 @@ static bool is_pow_of_two(int64_t x) {
   return (x & (x - 1)) == 0;
 }
 
-#ifdef __HIP_PLATFORM_HCC__
-    using cufft_size_type = int;
-#else
-    using cufft_size_type = long long int;
-#endif
+using cufft_size_type = long long int;
 
 using CuFFTDimVector = c10::SmallVector<cufft_size_type, at::kDimVectorStaticSize>;
 
@@ -140,7 +139,6 @@ struct CuFFTDataLayout {
 // e.g. if the input is cloned, this will be the resulting data layout
 // See NOTE [ cuFFT Embedded Strides ].
 inline CuFFTDataLayout cufft_simple_embed(IntArrayRef sizes, bool onesided) {
-  const auto signal_ndim = sizes.size() - 1;
   CuFFTDataLayout layout;
   layout.simple = true;
   layout.must_clone = false;
@@ -167,7 +165,7 @@ inline CuFFTDataLayout as_cufft_embed(IntArrayRef strides, IntArrayRef sizes, bo
 
   const auto last_dim_size = onesided ?
       sizes[signal_ndim] / 2 + 1 : sizes[signal_ndim];
-  const auto signal_numel = at::prod_intlist(sizes.slice(1, sizes.size() - 2)) * last_dim_size;
+  const auto signal_numel = c10::multiply_integers(sizes.slice(1, sizes.size() - 2)) * last_dim_size;
 
   // Zero stides are not allowed, even if the batch size is one.
   // If that happens just set a dummy case
@@ -202,7 +200,7 @@ inline CuFFTDataLayout as_cufft_embed(IntArrayRef strides, IntArrayRef sizes, bo
     layout.stride = strides[signal_ndim];
     // Determine if layout represents a simple embedding (contiguous data)
     layout.simple = [&] {
-      for (int64_t i = 1; i < signal_ndim - 1; ++i) {
+      for (const auto i : c10::irange(1, signal_ndim - 1)) {
         if (layout.embed[i] != sizes[i + 1]) {
           return false;
         }
@@ -255,7 +253,13 @@ public:
     // Since cuFFT has limited non-unit stride support and various constraints, we
     // use a flag to keep track throughout this function to see if we need to
     // input = input.clone();
+
+#if defined(USE_ROCM)
+    // clone input to avoid issues with hipfft clobering the input and failing tests
+    clone_input = true;
+#else
     clone_input = false;
+#endif
 
     // For half, base strides on the real part of real-to-complex and
     // complex-to-real transforms are not supported. Since our output is always
@@ -267,7 +271,7 @@ public:
                "cuFFT doesn't support signals of half type with compute "
                "capability less than SM_53, but the device containing input half "
                "tensor only has SM_", dev_prop->major, dev_prop->minor);
-      for (int64_t i = 0; i < signal_ndim; i++) {
+      for (const auto i : c10::irange(signal_ndim)) {
         TORCH_CHECK(is_pow_of_two(sizes[i + 1]),
             "cuFFT only supports dimensions whose sizes are powers of two when"
             " computing in half precision, but got a signal size of",
@@ -291,25 +295,6 @@ public:
     // See NOTE [ cuFFT Embedded Strides ] in native/cuda/SpectralOps.cu.
 
     const bool simple_layout = in_layout.simple && out_layout.simple;
-
-#ifdef __HIP_PLATFORM_HCC__
-    hipfftType exec_type = [&]{
-      if (dtype == kFloat) {
-        switch (fft_type) {
-          case CuFFTTransformType::C2C: return HIPFFT_C2C;
-          case CuFFTTransformType::R2C: return HIPFFT_R2C;
-          case CuFFTTransformType::C2R: return HIPFFT_C2R;
-        }
-      } else if (dtype == kDouble) {
-        switch (fft_type) {
-          case CuFFTTransformType::C2C: return HIPFFT_Z2Z;
-          case CuFFTTransformType::R2C: return HIPFFT_D2Z;
-          case CuFFTTransformType::C2R: return HIPFFT_Z2D;
-        }
-      }
-      TORCH_CHECK(false, "hipFFT doesn't support transforms of type: ", dtype);
-    }();
-#else
     cudaDataType itype, otype, exec_type;
     const auto complex_input = cufft_complex_input(fft_type);
     const auto complex_output = cufft_complex_output(fft_type);
@@ -328,7 +313,6 @@ public:
     } else {
       TORCH_CHECK(false, "cuFFT doesn't support tensor of type: ", dtype);
     }
-#endif
 
     // disable auto allocation of workspace to use THC allocator
     CUFFT_CHECK(cufftSetAutoAllocation(plan(), /* autoAllocate */ 0));
@@ -342,29 +326,15 @@ public:
       // by assuming istride = ostride = 1.
       //
       // See NOTE [ cuFFT Embedded Strides ] in native/cuda/SpectralOps.cu.
-#ifdef __HIP_PLATFORM_HCC__
-      CUFFT_CHECK(hipfftMakePlanMany(plan(), signal_ndim, signal_sizes.data(),
-        /* inembed */ nullptr, /* base_istride */ 1, /* idist */ 1,
-        /* onembed */ nullptr, /* base_ostride */ 1, /* odist */ 1,
-        exec_type, batch, &ws_size_t));
-#else
       CUFFT_CHECK(cufftXtMakePlanMany(plan(), signal_ndim, signal_sizes.data(),
         /* inembed */ nullptr, /* base_istride */ 1, /* idist */ 1, itype,
         /* onembed */ nullptr, /* base_ostride */ 1, /* odist */ 1, otype,
         batch, &ws_size_t, exec_type));
-#endif
     } else {
-#ifdef __HIP_PLATFORM_HCC__
-      CUFFT_CHECK(hipfftMakePlanMany(plan(), signal_ndim, signal_sizes.data(),
-        in_layout.embed.data(), in_layout.stride, in_layout.dist,
-        out_layout.embed.data(), out_layout.stride, out_layout.dist,
-        exec_type, batch, &ws_size_t));
-#else
       CUFFT_CHECK(cufftXtMakePlanMany(plan(), signal_ndim, signal_sizes.data(),
             in_layout.embed.data(), in_layout.stride, in_layout.dist, itype,
             out_layout.embed.data(), out_layout.stride, out_layout.dist, otype,
             batch, &ws_size_t, exec_type));
-#endif
     }
     ws_size = static_cast<int64_t>(ws_size_t);
   }
@@ -384,19 +354,19 @@ private:
   ScalarType value_type_;
 };
 
-#if CUDA_VERSION < 10000
+#if defined(USE_ROCM)
   // Note that the max plan number for CUDA version < 10 has to be 1023
   // due to a bug that fails on the 1024th plan
-  constexpr size_t CUFFT_MAX_PLAN_NUM = 1023;
-  constexpr size_t CUFFT_DEFAULT_CACHE_SIZE = CUFFT_MAX_PLAN_NUM;
+  constexpr int64_t CUFFT_MAX_PLAN_NUM = 1023;
+  constexpr int64_t CUFFT_DEFAULT_CACHE_SIZE = CUFFT_MAX_PLAN_NUM;
 #else
-  constexpr size_t CUFFT_MAX_PLAN_NUM = std::numeric_limits<size_t>::max();
+  constexpr int64_t CUFFT_MAX_PLAN_NUM = std::numeric_limits<int64_t>::max();
   // The default max cache size chosen for CUDA version > 10 is arbitrary.
   // This number puts a limit on how big of a plan cache should we maintain by
   // default. Users can always configure it via cufft_set_plan_cache_max_size.
-  constexpr size_t CUFFT_DEFAULT_CACHE_SIZE = 4096;
+  constexpr int64_t CUFFT_DEFAULT_CACHE_SIZE = 4096;
 #endif
-static_assert(CUFFT_MAX_PLAN_NUM >= 0 && CUFFT_MAX_PLAN_NUM <= std::numeric_limits<size_t>::max(),
+static_assert(0 <= CUFFT_MAX_PLAN_NUM && CUFFT_MAX_PLAN_NUM <= std::numeric_limits<int64_t>::max(),
               "CUFFT_MAX_PLAN_NUM not in size_t range");
 static_assert(CUFFT_DEFAULT_CACHE_SIZE >= 0 && CUFFT_DEFAULT_CACHE_SIZE <= CUFFT_MAX_PLAN_NUM,
               "CUFFT_DEFAULT_CACHE_SIZE not in [0, CUFFT_MAX_PLAN_NUM] range");
@@ -516,9 +486,9 @@ private:
 // native function counterparts (at native/SpectralOps.cpp), i.e.,
 // _cufft_get_plan_cache_max_size, _cufft_set_plan_cache_max_size
 // _cufft_get_plan_cache_size, and _cufft_clear_plan_cache.
-int64_t cufft_get_plan_cache_max_size_impl(int64_t device_index);
-void cufft_set_plan_cache_max_size_impl(int64_t device_index, int64_t max_size);
-int64_t cufft_get_plan_cache_size_impl(int64_t device_index);
-void cufft_clear_plan_cache_impl(int64_t device_index);
+int64_t cufft_get_plan_cache_max_size_impl(DeviceIndex device_index);
+void cufft_set_plan_cache_max_size_impl(DeviceIndex device_index, int64_t max_size);
+int64_t cufft_get_plan_cache_size_impl(DeviceIndex device_index);
+void cufft_clear_plan_cache_impl(DeviceIndex device_index);
 
-}}} // namespace at::native::detail
+} // namespace at::native::detail
