@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+# flake8: noqa: F821
 
 import importlib
 import logging
@@ -48,7 +49,6 @@ def pip_install(package):
 
 # Disable the flake warnings for the imports. Flake8 does not provide a way to
 # disable just warning for the entire file. Disabling flake8 entirely.
-# flake8: noqa
 imports = [
     "AlbertForPreTraining",
     "AutoConfig",
@@ -111,7 +111,7 @@ BATCH_SIZE_KNOWN_MODELS = {}
 # Get the list of models and their batch sizes
 MODELS_FILENAME = os.path.join(os.path.dirname(__file__), "huggingface_models_list.txt")
 assert os.path.exists(MODELS_FILENAME)
-with open(MODELS_FILENAME, "r") as fh:
+with open(MODELS_FILENAME) as fh:
     lines = fh.readlines()
     lines = [line.rstrip() for line in lines]
     for line in lines:
@@ -166,7 +166,7 @@ def get_sequence_length(model_cls, model_name):
         seq_length = 10000  # NB: a more realistic size is 155136
     else:
         log.info(
-            f"Sequence Length not defined for {model_name}. Choosing 128 arbitrarily"
+            f"Sequence Length not defined for {model_name}. Choosing 128 arbitrarily"  # noqa: G004
         )
         seq_length = 128
     return seq_length
@@ -204,22 +204,16 @@ def generate_inputs_for_model(
 
     input_dict = {"input_ids": input}
 
-    if (
-        model_name.startswith("T5")
-        or model_name.startswith("M2M100")
-        or model_name.startswith("MT5")
-        or model_cls
-        in [
-            BlenderbotModel,
-            BlenderbotSmallModel,
-            BlenderbotForConditionalGeneration,
-            BlenderbotSmallForConditionalGeneration,
-            PegasusModel,
-            PegasusForConditionalGeneration,
-            MarianModel,
-            MarianMTModel,
-        ]
-    ):
+    if model_name.startswith(("T5", "M2M100", "MT5")) or model_cls in [
+        BlenderbotModel,
+        BlenderbotSmallModel,
+        BlenderbotForConditionalGeneration,
+        BlenderbotSmallForConditionalGeneration,
+        PegasusModel,
+        PegasusForConditionalGeneration,
+        MarianModel,
+        MarianMTModel,
+    ]:
         input_dict["decoder_input_ids"] = input
 
     if model_name.startswith("Lxmert"):
@@ -251,11 +245,8 @@ def generate_inputs_for_model(
                 device, 0, seq_length, (bs,)
             )
             input_dict["end_positions"] = rand_int_tensor(device, 0, seq_length, (bs,))
-        elif (
-            model_name.endswith("MaskedLM")
-            or model_name.endswith("HeadModel")
-            or model_name.endswith("CausalLM")
-            or model_name.endswith("DoubleHeadsModel")
+        elif model_name.endswith(
+            ("MaskedLM", "HeadModel", "CausalLM", "DoubleHeadsModel")
         ):
             input_dict["labels"] = rand_int_tensor(
                 device, 0, vocab_size, (bs, seq_length)
@@ -369,7 +360,10 @@ class HuggingfaceRunner(BenchmarkRunner):
         return self._skip["control_flow"]
 
     def use_larger_multiplier_for_smaller_tensor(self, name):
-        return name in ["ElectraForQuestionAnswering"]
+        return name in [
+            "ElectraForQuestionAnswering",
+            "MegatronBertForQuestionAnswering",
+        ]
 
     def _get_model_cls_and_config(self, model_name):
         if model_name not in EXTRA_MODELS:
@@ -426,7 +420,7 @@ class HuggingfaceRunner(BenchmarkRunner):
         elif batch_size is None:
             batch_size_default = 16
             log.info(
-                f"Batch size not specified for {model_name}. Setting batch_size=16"
+                f"Batch size not specified for {model_name}. Setting batch_size=16"  # noqa: G004
             )
 
         if batch_size is None:
@@ -435,7 +429,7 @@ class HuggingfaceRunner(BenchmarkRunner):
             if model_name in batch_size_divisors:
                 batch_size = max(int(batch_size / batch_size_divisors[model_name]), 1)
                 log.info(
-                    f"Running smaller batch size={batch_size} for {model_name}, orig batch_size={batch_size_default}"
+                    f"Running smaller batch size={batch_size} for {model_name}, orig batch_size={batch_size_default}"  # noqa: G004
                 )
 
         example_inputs = generate_inputs_for_model(
@@ -471,8 +465,8 @@ class HuggingfaceRunner(BenchmarkRunner):
             if index < start or index >= end:
                 continue
             if (
-                not re.search("|".join(args.filter), model_name, re.I)
-                or re.search("|".join(args.exclude), model_name, re.I)
+                not re.search("|".join(args.filter), model_name, re.IGNORECASE)
+                or re.search("|".join(args.exclude), model_name, re.IGNORECASE)
                 or model_name in args.exclude_exact
                 or model_name in self.skip_models
             ):
@@ -533,7 +527,7 @@ class HuggingfaceRunner(BenchmarkRunner):
         self.grad_scaler.scale(loss).backward()
         self.optimizer_step()
         if collect_outputs:
-            return collect_results(mod, pred, loss, cloned_inputs)
+            return collect_results(mod, None, loss, cloned_inputs)
         return None
 
 
@@ -618,7 +612,7 @@ def refresh_model_names_and_batch_sizes():
                 + [f"--output={MODELS_FILENAME}"]
             )
         except subprocess.SubprocessError:
-            log.warning(f"Failed to find suitable batch size for {model_name}")
+            log.warning(f"Failed to find suitable batch size for {model_name}")  # noqa: G004
 
 
 def huggingface_main():
