@@ -1,10 +1,11 @@
-#include <c10/util/typeid.h>
 #include <c10/core/DefaultDtype.h>
+#include <c10/util/typeid.h>
 
 namespace c10 {
 static auto default_dtype = caffe2::TypeMeta::Make<float>();
 static auto default_dtype_as_scalartype = default_dtype.toScalarType();
-static auto default_complex_dtype = caffe2::TypeMeta::Make<c10::complex<float>>();
+static auto default_complex_dtype =
+    caffe2::TypeMeta::Make<c10::complex<float>>();
 
 void set_default_dtype(caffe2::TypeMeta dtype) {
   default_dtype = dtype;
