@@ -1,21 +1,31 @@
+# Owner(s): ["module: unknown"]
+
+import collections
 import json
 import os
 import re
 import textwrap
 import timeit
-from typing import Any, List, Tuple
 import unittest
+from typing import Any
+
+import expecttest
+import numpy as np
 
 import torch
 import torch.utils.benchmark as benchmark_utils
-from torch.testing._internal.common_utils import TestCase, run_tests, IS_SANDCASTLE, IS_WINDOWS, slowTest
-from torch.testing._internal import expecttest
-import numpy as np
+from torch.testing._internal.common_utils import (
+    IS_SANDCASTLE,
+    IS_WINDOWS,
+    run_tests,
+    slowTest,
+    TEST_WITH_ASAN,
+    TestCase,
+)
 
 
 CALLGRIND_ARTIFACTS: str = os.path.join(
-    os.path.split(os.path.abspath(__file__))[0],
-    "callgrind_artifacts.json"
+    os.path.split(os.path.abspath(__file__))[0], "callgrind_artifacts.json"
 )
 
 
@@ -30,13 +40,13 @@ def generate_callgrind_artifacts() -> None:
     """
     print("Regenerating callgrind artifact.")
 
-    stats_no_data = benchmark_utils.Timer(
-        "y = torch.ones(())"
-    ).collect_callgrind(number=1000)
+    stats_no_data = benchmark_utils.Timer("y = torch.ones(())").collect_callgrind(
+        number=1000
+    )
 
-    stats_with_data = benchmark_utils.Timer(
-        "y = torch.ones((1,))"
-    ).collect_callgrind(number=1000)
+    stats_with_data = benchmark_utils.Timer("y = torch.ones((1,))").collect_callgrind(
+        number=1000
+    )
 
     user = os.getenv("USER")
 
@@ -52,11 +62,14 @@ def generate_callgrind_artifacts() -> None:
         "ones_with_data_exclusive": to_entry(stats_with_data.stmt_exclusive_stats),
     }
 
-    with open(CALLGRIND_ARTIFACTS, "wt") as f:
+    with open(CALLGRIND_ARTIFACTS, "w") as f:
         json.dump(artifacts, f, indent=4)
 
 
-def load_callgrind_artifacts() -> Tuple[benchmark_utils.CallgrindStats, benchmark_utils.CallgrindStats]:
+def load_callgrind_artifacts() -> tuple[
+    benchmark_utils.CallgrindStats,
+    benchmark_utils.CallgrindStats,
+]:
     """Hermetic artifact to unit test Callgrind wrapper.
 
     In addition to collecting counts, this wrapper provides some facilities for
@@ -67,16 +80,15 @@ def load_callgrind_artifacts() -> Tuple[benchmark_utils.CallgrindStats, benchmar
     testing are stored in raw string form for easier inspection and to avoid
     baking any implementation details into the artifact itself.
     """
-    with open(CALLGRIND_ARTIFACTS, "rt") as f:
+    with open(CALLGRIND_ARTIFACTS) as f:
         artifacts = json.load(f)
 
     pattern = re.compile(r"^\s*([0-9]+)\s(.+)$")
 
     def to_function_counts(
-        count_strings: List[str],
-        inclusive: bool
+        count_strings: list[str], inclusive: bool
     ) -> benchmark_utils.FunctionCounts:
-        data: List[benchmark_utils.FunctionCount] = []
+        data: list[benchmark_utils.FunctionCount] = []
         for cs in count_strings:
             # Storing entries as f"{c} {fn}" rather than [c, fn] adds some work
             # reviving the artifact, but it makes the json much easier to read.
@@ -86,8 +98,8 @@ def load_callgrind_artifacts() -> Tuple[benchmark_utils.CallgrindStats, benchmar
             data.append(benchmark_utils.FunctionCount(count=int(c), function=fn))
 
         return benchmark_utils.FunctionCounts(
-            tuple(sorted(data, reverse=True)),
-            inclusive=inclusive)
+            tuple(sorted(data, reverse=True)), inclusive=inclusive
+        )
 
     baseline_inclusive = to_function_counts(artifacts["baseline_inclusive"], True)
     baseline_exclusive = to_function_counts(artifacts["baseline_exclusive"], False)
@@ -98,8 +110,13 @@ def load_callgrind_artifacts() -> Tuple[benchmark_utils.CallgrindStats, benchmar
         built_with_debug_symbols=True,
         baseline_inclusive_stats=baseline_inclusive,
         baseline_exclusive_stats=baseline_exclusive,
-        stmt_inclusive_stats=to_function_counts(artifacts["ones_no_data_inclusive"], True),
-        stmt_exclusive_stats=to_function_counts(artifacts["ones_no_data_exclusive"], False),
+        stmt_inclusive_stats=to_function_counts(
+            artifacts["ones_no_data_inclusive"], True
+        ),
+        stmt_exclusive_stats=to_function_counts(
+            artifacts["ones_no_data_exclusive"], False
+        ),
+        stmt_callgrind_out=None,
     )
 
     stats_with_data = benchmark_utils.CallgrindStats(
@@ -108,8 +125,13 @@ def load_callgrind_artifacts() -> Tuple[benchmark_utils.CallgrindStats, benchmar
         built_with_debug_symbols=True,
         baseline_inclusive_stats=baseline_inclusive,
         baseline_exclusive_stats=baseline_exclusive,
-        stmt_inclusive_stats=to_function_counts(artifacts["ones_with_data_inclusive"], True),
-        stmt_exclusive_stats=to_function_counts(artifacts["ones_with_data_exclusive"], False),
+        stmt_inclusive_stats=to_function_counts(
+            artifacts["ones_with_data_inclusive"], True
+        ),
+        stmt_exclusive_stats=to_function_counts(
+            artifacts["ones_with_data_exclusive"], False
+        ),
+        stmt_callgrind_out=None,
     )
 
     return stats_no_data, stats_with_data
@@ -122,14 +144,12 @@ class MyModule(torch.nn.Module):
 
 class TestBenchmarkUtils(TestCase):
     def regularizeAndAssertExpectedInline(
-        self, x: Any,
-        expect: str,
-        indent: int = 12
+        self, x: Any, expect: str, indent: int = 12
     ) -> None:
         x_str: str = re.sub(
             "object at 0x[0-9a-fA-F]+>",
             "object at 0xXXXXXXXXXXXX>",
-            x if isinstance(x, str) else repr(x)
+            x if isinstance(x, str) else repr(x),
         )
         if "\n" in x_str:
             # Indent makes the reference align at the call site.
@@ -152,23 +172,47 @@ class TestBenchmarkUtils(TestCase):
         median = timer.adaptive_autorange(threshold=0.5).median
 
         # Test that multi-line statements work properly.
-        median = benchmark_utils.Timer(
-            stmt="""
+        median = (
+            benchmark_utils.Timer(
+                stmt="""
                 with torch.no_grad():
                     y = x + 1""",
-            setup="""
+                setup="""
                 x = torch.ones((1,), requires_grad=True)
                 for _ in range(5):
                     x = x + 1.0""",
-        ).timeit(5).median
+            )
+            .timeit(5)
+            .median
+        )
         self.assertIsInstance(sample, float)
 
     @slowTest
     @unittest.skipIf(IS_SANDCASTLE, "C++ timing is OSS only.")
+    @unittest.skipIf(True, "Failing on clang, see 74398")
+    def test_timer_tiny_fast_snippet(self):
+        timer = benchmark_utils.Timer(
+            "auto x = 1;(void)x;",
+            timer=timeit.default_timer,
+            language=benchmark_utils.Language.CPP,
+        )
+        median = timer.blocked_autorange().median
+        self.assertIsInstance(median, float)
+
+    @slowTest
+    @unittest.skipIf(IS_SANDCASTLE, "C++ timing is OSS only.")
+    @unittest.skipIf(True, "Failing on clang, see 74398")
     def test_cpp_timer(self):
         timer = benchmark_utils.Timer(
-            "torch::Tensor y = x + 1;",
+            """
+                #ifndef TIMER_GLOBAL_CHECK
+                static_assert(false);
+                #endif
+
+                torch::Tensor y = x + 1;
+            """,
             setup="torch::Tensor x = torch::empty({1});",
+            global_setup="#define TIMER_GLOBAL_CHECK",
             timer=timeit.default_timer,
             language=benchmark_utils.Language.CPP,
         )
@@ -191,22 +235,22 @@ class TestBenchmarkUtils(TestCase):
 
         def __init__(self, stmt, setup, timer, globals):
             self._random_state = np.random.RandomState(seed=self._seed)
-            self._mean_cost = {k: v for k, v in self._function_costs}[stmt]
+            self._mean_cost = dict(self._function_costs)[stmt]
 
         def sample(self, mean, noise_level):
             return max(self._random_state.normal(mean, mean * noise_level), 5e-9)
 
         def timeit(self, number):
-            return sum([
-                # First timer invocation
-                self.sample(self._timer_cost, self._timer_noise_level),
-
-                # Stmt body
-                self.sample(self._mean_cost * number, self._function_noise_level),
-
-                # Second timer invocation
-                self.sample(self._timer_cost, self._timer_noise_level),
-            ])
+            return sum(
+                [
+                    # First timer invocation
+                    self.sample(self._timer_cost, self._timer_noise_level),
+                    # Stmt body
+                    self.sample(self._mean_cost * number, self._function_noise_level),
+                    # Second timer invocation
+                    self.sample(self._timer_cost, self._timer_noise_level),
+                ]
+            )
 
     def test_adaptive_timer(self):
         class MockTimer(benchmark_utils.Timer):
@@ -220,7 +264,6 @@ class TestBenchmarkUtils(TestCase):
             _function_costs = (
                 self._MockTimer._function_costs[0],
                 self._MockTimer._function_costs[1],
-
                 # GPU should be faster once there is enough work.
                 ("expensive_fn()", 5e-6),
             )
@@ -236,7 +279,7 @@ class TestBenchmarkUtils(TestCase):
             pass
               Median: 7.98 ns
               IQR:    0.52 ns (7.74 to 8.26)
-              125 measurements, 10000000 runs per measurement, 1 thread"""
+              125 measurements, 10000000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -246,11 +289,11 @@ class TestBenchmarkUtils(TestCase):
             pass
               Median: 7.86 ns
               IQR:    0.71 ns (7.63 to 8.34)
-              6 measurements, 1000000 runs per measurement, 1 thread"""
+              6 measurements, 1000000 runs per measurement, 1 thread""",
         )
 
         # Check against strings so we can reuse expect infra.
-        self.regularizeAndAssertExpectedInline(m.mean, """8.001365835795602e-09""")
+        self.regularizeAndAssertExpectedInline(m.mean, """8.0013658357956e-09""")
         self.regularizeAndAssertExpectedInline(m.median, """7.983151323215967e-09""")
         self.regularizeAndAssertExpectedInline(len(m.times), """125""")
         self.regularizeAndAssertExpectedInline(m.number_per_run, """10000000""")
@@ -262,7 +305,7 @@ class TestBenchmarkUtils(TestCase):
             cheap_fn()
               Median: 3.98 us
               IQR:    0.27 us (3.85 to 4.12)
-              252 measurements, 10000 runs per measurement, 1 thread"""
+              252 measurements, 10000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -272,7 +315,7 @@ class TestBenchmarkUtils(TestCase):
             cheap_fn()
               Median: 4.16 us
               IQR:    0.22 us (4.04 to 4.26)
-              4 measurements, 1000 runs per measurement, 1 thread"""
+              4 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -282,7 +325,7 @@ class TestBenchmarkUtils(TestCase):
             expensive_fn()
               Median: 19.97 us
               IQR:    1.35 us (19.31 to 20.65)
-              501 measurements, 1000 runs per measurement, 1 thread"""
+              501 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -292,7 +335,7 @@ class TestBenchmarkUtils(TestCase):
             expensive_fn()
               Median: 20.79 us
               IQR:    1.09 us (20.20 to 21.29)
-              4 measurements, 1000 runs per measurement, 1 thread"""
+              4 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -302,7 +345,7 @@ class TestBenchmarkUtils(TestCase):
             pass
               Median: 7.92 ns
               IQR:    0.43 ns (7.75 to 8.17)
-              13 measurements, 100000000 runs per measurement, 1 thread"""
+              13 measurements, 100000000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -312,7 +355,7 @@ class TestBenchmarkUtils(TestCase):
             pass
               Median: 7.75 ns
               IQR:    0.57 ns (7.56 to 8.13)
-              4 measurements, 10000000 runs per measurement, 1 thread"""
+              4 measurements, 10000000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -322,7 +365,7 @@ class TestBenchmarkUtils(TestCase):
             cheap_fn()
               Median: 4.04 us
               IQR:    0.30 us (3.90 to 4.19)
-              25 measurements, 100000 runs per measurement, 1 thread"""
+              25 measurements, 100000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -332,7 +375,7 @@ class TestBenchmarkUtils(TestCase):
             cheap_fn()
               Median: 4.09 us
               IQR:    0.38 us (3.90 to 4.28)
-              4 measurements, 100000 runs per measurement, 1 thread"""
+              4 measurements, 100000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -342,7 +385,7 @@ class TestBenchmarkUtils(TestCase):
             expensive_fn()
               Median: 4.98 us
               IQR:    0.31 us (4.83 to 5.13)
-              20 measurements, 100000 runs per measurement, 1 thread"""
+              20 measurements, 100000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -352,7 +395,7 @@ class TestBenchmarkUtils(TestCase):
             expensive_fn()
               Median: 5.01 us
               IQR:    0.28 us (4.87 to 5.15)
-              4 measurements, 10000 runs per measurement, 1 thread"""
+              4 measurements, 10000 runs per measurement, 1 thread""",
         )
 
         # Make sure __repr__ is reasonable for
@@ -373,7 +416,7 @@ class TestBenchmarkUtils(TestCase):
 
               Median: 10.06 us
               IQR:    0.54 us (9.73 to 10.27)
-              20 measurements, 1000 runs per measurement, 1 thread"""
+              20 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -386,7 +429,7 @@ class TestBenchmarkUtils(TestCase):
 
               Median: 10.06 us
               IQR:    0.54 us (9.73 to 10.27)
-              20 measurements, 1000 runs per measurement, 1 thread"""
+              20 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -400,7 +443,7 @@ class TestBenchmarkUtils(TestCase):
             x + 1 (no grad): scalar_add
               Median: 10.06 us
               IQR:    0.54 us (9.73 to 10.27)
-              20 measurements, 1000 runs per measurement, 1 thread"""
+              20 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -418,7 +461,7 @@ class TestBenchmarkUtils(TestCase):
             setup: setup_fn()
               Median: 10.06 us
               IQR:    0.54 us (9.73 to 10.27)
-              20 measurements, 1000 runs per measurement, 1 thread"""
+              20 measurements, 1000 runs per measurement, 1 thread""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -446,34 +489,34 @@ class TestBenchmarkUtils(TestCase):
 
               Median: 10.06 us
               IQR:    0.54 us (9.73 to 10.27)
-              20 measurements, 1000 runs per measurement, 16 threads"""
+              20 measurements, 1000 runs per measurement, 16 threads""",
         )
 
     @slowTest
     @unittest.skipIf(IS_WINDOWS, "Valgrind is not supported on Windows.")
     @unittest.skipIf(IS_SANDCASTLE, "Valgrind is OSS only.")
+    @unittest.skipIf(TEST_WITH_ASAN, "fails on asan")
     def test_collect_callgrind(self):
         with self.assertRaisesRegex(
             ValueError,
             r"`collect_callgrind` requires that globals be wrapped "
-            r"in `CopyIfCallgrind` so that serialization is explicit."
+            r"in `CopyIfCallgrind` so that serialization is explicit.",
         ):
-            benchmark_utils.Timer(
-                "pass",
-                globals={"x": 1}
-            ).collect_callgrind(collect_baseline=False)
+            benchmark_utils.Timer("pass", globals={"x": 1}).collect_callgrind(
+                collect_baseline=False
+            )
 
         with self.assertRaisesRegex(
             # Subprocess raises AttributeError (from pickle),
             # _ValgrindWrapper re-raises as generic OSError.
-            OSError, "AttributeError: Can't get attribute 'MyModule'"
+            OSError,
+            "AttributeError: Can't get attribute 'MyModule'",
         ):
             benchmark_utils.Timer(
                 "model(1)",
-                globals={"model": benchmark_utils.CopyIfCallgrind(MyModule())}
+                globals={"model": benchmark_utils.CopyIfCallgrind(MyModule())},
             ).collect_callgrind(collect_baseline=False)
 
-
         @torch.jit.script
         def add_one(x):
             return x + 1
@@ -490,27 +533,50 @@ class TestBenchmarkUtils(TestCase):
                     import sys
                     sys.path.append({repr(os.path.split(os.path.abspath(__file__))[0])})
                     from test_benchmark_utils import MyModule
-                    """
-                )
-            }
+                    """,
+                ),
+            },
         )
 
-        # Don't collect baseline to speed up unit test by ~30 seconds.
-        stats = timer.collect_callgrind(number=1000, collect_baseline=False)
+        stats = timer.collect_callgrind(number=1000)
         counts = stats.counts(denoise=False)
 
         self.assertIsInstance(counts, int)
         self.assertGreater(counts, 0)
 
-        from torch.utils.benchmark.utils.valgrind_wrapper.timer_interface import wrapper_singleton
+        # There is some jitter with the allocator, so we use a simpler task to
+        # test reproducibility.
+        timer = benchmark_utils.Timer(
+            "x += 1",
+            setup="x = torch.ones((1,))",
+        )
+
+        stats = timer.collect_callgrind(number=1000, repeats=20)
+        assert isinstance(stats, tuple)
+
+        # Check that the repeats are at least somewhat repeatable. (within 10 instructions per iter)
+        counts = collections.Counter(
+            [s.counts(denoise=True) // 10_000 * 10_000 for s in stats]
+        )
+        self.assertGreater(
+            max(counts.values()),
+            1,
+            f"Every instruction count total was unique: {counts}",
+        )
+
+        from torch.utils.benchmark.utils.valgrind_wrapper.timer_interface import (
+            wrapper_singleton,
+        )
+
         self.assertIsNone(
             wrapper_singleton()._bindings_module,
-            "JIT'd bindings are only for back testing."
+            "JIT'd bindings are only for back testing.",
         )
 
     @slowTest
     @unittest.skipIf(IS_WINDOWS, "Valgrind is not supported on Windows.")
     @unittest.skipIf(IS_SANDCASTLE, "Valgrind is OSS only.")
+    @unittest.skipIf(True, "Failing on clang, see 74398")
     def test_collect_cpp_callgrind(self):
         timer = benchmark_utils.Timer(
             "x += 1;",
@@ -518,28 +584,38 @@ class TestBenchmarkUtils(TestCase):
             timer=timeit.default_timer,
             language="c++",
         )
-        stats = [
-            timer.collect_callgrind()
-            for _ in range(3)
-        ]
+        stats = [timer.collect_callgrind() for _ in range(3)]
         counts = [s.counts() for s in stats]
 
-        self.assertGreater(
-            min(counts), 0, "No stats were collected")
+        self.assertGreater(min(counts), 0, "No stats were collected")
         self.assertEqual(
-            min(counts), max(counts), "C++ Callgrind should be deterministic")
+            min(counts), max(counts), "C++ Callgrind should be deterministic"
+        )
 
         for s in stats:
             self.assertEqual(
-                s.counts(denoise=True), s.counts(denoise=False),
-                "De-noising should not apply to C++.")
+                s.counts(denoise=True),
+                s.counts(denoise=False),
+                "De-noising should not apply to C++.",
+            )
+
+        stats = timer.collect_callgrind(number=1000, repeats=20)
+        assert isinstance(stats, tuple)
+
+        # NB: Unlike the example above, there is no expectation that all
+        #     repeats will be identical.
+        counts = collections.Counter(
+            [s.counts(denoise=True) // 10_000 * 10_000 for s in stats]
+        )
+        self.assertGreater(max(counts.values()), 1, repr(counts))
 
     def test_manipulate_callgrind_stats(self):
         stats_no_data, stats_with_data = load_callgrind_artifacts()
 
         # Mock `torch.set_printoptions(linewidth=160)`
         wide_linewidth = benchmark_utils.FunctionCounts(
-            stats_no_data.stats(inclusive=False)._data, False, 160)
+            stats_no_data.stats(inclusive=False)._data, False, _linewidth=160
+        )
 
         for l in repr(wide_linewidth).splitlines(keepends=False):
             self.assertLessEqual(len(l), 160)
@@ -547,10 +623,12 @@ class TestBenchmarkUtils(TestCase):
         self.assertEqual(
             # `delta` is just a convenience method.
             stats_with_data.delta(stats_no_data)._data,
-            (stats_with_data.stats() - stats_no_data.stats())._data
+            (stats_with_data.stats() - stats_no_data.stats())._data,
         )
 
-        deltas = stats_with_data.as_standardized().delta(stats_no_data.as_standardized())
+        deltas = stats_with_data.as_standardized().delta(
+            stats_no_data.as_standardized()
+        )
 
         def custom_transforms(fn: str):
             fn = re.sub(re.escape("/usr/include/c++/8/bits/"), "", fn)
@@ -590,45 +668,45 @@ class TestBenchmarkUtils(TestCase):
               192000  build/../c10/core/Device.h:c10:: ... epos/pytorch/torch/lib/libc10.so]
               169855  ???:_int_malloc [/usr/lib64/libc-2.28.so]
               154000  build/../c10/core/TensorOptions. ... ytorch/torch/lib/libtorch_cpu.so]
-              147167  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
+              148561  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
               135000  ???:malloc [/usr/lib64/libc-2.28.so]
                  ...
-                 -62  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                 -63  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                 -70  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                 -74  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                 -85  /home/test_user/miniconda3/envs/ ... rch/torch/lib/libtorch_python.so]
-                 -95  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                -104  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                -134  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
-                -180  /tmp/build/80754af9/python_15996 ... da3/envs/throwaway/bin/python3.6]
+                2000  /usr/include/c++/8/ext/new_allocator.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/stl_vect ... *, _object*, _object*, _object**)
+                2000  /usr/include/c++/8/bits/stl_vect ... rningHandler::~PyWarningHandler()
+                2000  /usr/include/c++/8/bits/stl_vect ... ject*, _object*, _object**, bool)
+                2000  /usr/include/c++/8/bits/stl_algobase.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/shared_p ... ad_accumulator(at::Tensor const&)
+                2000  /usr/include/c++/8/bits/move.h:c ... te<c10::AutogradMetaInterface> >)
+                2000  /usr/include/c++/8/bits/atomic_b ... DispatchKey&&, caffe2::TypeMeta&)
+                2000  /usr/include/c++/8/array:at::Ten ... , at::Tensor&, c10::Scalar) const
 
-            Total: 8863284""",
+            Total: 8869966""",
         )
 
         self.regularizeAndAssertExpectedInline(
             stats_no_data.stats(inclusive=True),
             """\
             <torch.utils.benchmark.utils.valgrind_wrapper.timer_interface.FunctionCounts object at 0xXXXXXXXXXXXX>
-              8952420  ???:0x0000000000001050 [/usr/lib64/ld-2.28.so]
-              8952420  ???:(below main) [/usr/lib64/libc-2.28.so]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-              8952420  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  ???:0x0000000000001050 [/usr/lib64/ld-2.28.so]
+              8959166  ???:(below main) [/usr/lib64/libc-2.28.so]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+              8959166  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
                   ...
-                 -195  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                 -196  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                 -207  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                 -261  /home/test_user/miniconda3/envs/ ... ch/torch/lib/libtorch_python.so]
-                 -561  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                 -789  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                 -881  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                -1196  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
-                -1206  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]""",
+                92821  /tmp/build/80754af9/python_15996 ... a3/envs/throwaway/bin/python3.6]
+                91000  build/../torch/csrc/tensor/pytho ... ch/torch/lib/libtorch_python.so]
+                91000  /data/users/test_user/repos/pyto ... nsors::get_default_scalar_type()
+                90090  ???:pthread_mutex_lock [/usr/lib64/libpthread-2.28.so]
+                90000  build/../c10/core/TensorImpl.h:c ... ch/torch/lib/libtorch_python.so]
+                90000  build/../aten/src/ATen/record_fu ... torch/torch/lib/libtorch_cpu.so]
+                90000  /data/users/test_user/repos/pyto ... uard(std::optional<c10::Device>)
+                90000  /data/users/test_user/repos/pyto ... ersionCounter::~VersionCounter()
+                88000  /data/users/test_user/repos/pyto ... ratorKernel*, at::Tensor const&)""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -642,20 +720,20 @@ class TestBenchmarkUtils(TestCase):
               192000  build/../c10/core/Device.h:c10::Device::validate() [/data/users/test_user/repos/pytorch/torch/lib/libc10.so]
               169855  ???:_int_malloc [/usr/lib64/libc-2.28.so]
               154000  build/../c10/core/TensorOptions.h:c10::TensorOptions::merge_in(c10::Tens ... ns) const [/data/users/test_user/repos/pytorch/torch/lib/libtorch_cpu.so]
-              147167  /tmp/build/80754af9/python_1599604603603/work/Python/ceval.c:_PyEval_EvalFrameDefault [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
+              148561  /tmp/build/80754af9/python_1599604603603/work/Python/ceval.c:_PyEval_EvalFrameDefault [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
               135000  ???:malloc [/usr/lib64/libc-2.28.so]
                  ...
-                 -62  /tmp/build/80754af9/python_1599604603603/work/Objects/abstract.c:PyNumber_Subtract [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                 -63  /tmp/build/80754af9/python_1599604603603/work/Objects/longobject.c:long_richcompare [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                 -70  /tmp/build/80754af9/python_1599604603603/work/Objects/abstract.c:_PyObject_FastCallDict [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                 -74  /tmp/build/80754af9/python_1599604603603/work/Python/pytime.c:_PyTime_FromSecondsObject [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                 -85  /home/test_user/miniconda3/envs/throwaway/include/pybind11/pybind11.h:py ... ject*) [/data/users/test_user/repos/pytorch/torch/lib/libtorch_python.so]
-                 -95  /tmp/build/80754af9/python_1599604603603/work/Objects/rangeobject.c:rangeiter_next [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                -104  /tmp/build/80754af9/python_1599604603603/work/Objects/object.c:PyObject_RichCompare [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                -134  /tmp/build/80754af9/python_1599604603603/work/Objects/rangeobject.c:range_new [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
-                -180  /tmp/build/80754af9/python_1599604603603/work/Objects/longobject.c:PyLong_FromLong [/home/test_user/miniconda3/envs/throwaway/bin/python3.6]
+                2000  /usr/include/c++/8/ext/new_allocator.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/stl_vector.h:torch::PythonArgParser::raw_parse(_object*, _object*, _object*, _object**)
+                2000  /usr/include/c++/8/bits/stl_vector.h:torch::PyWarningHandler::~PyWarningHandler()
+                2000  /usr/include/c++/8/bits/stl_vector.h:torch::FunctionSignature::parse(_object*, _object*, _object*, _object**, bool)
+                2000  /usr/include/c++/8/bits/stl_algobase.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/shared_ptr_base.h:torch::autograd::impl::try_get_grad_accumulator(at::Tensor const&)
+                2000  /usr/include/c++/8/bits/move.h:c10::TensorImpl::set_autograd_meta(std::u ... AutogradMetaInterface, std::default_delete<c10::AutogradMetaInterface> >)
+                2000  /usr/include/c++/8/bits/atomic_base.h:at::Tensor at::detail::make_tensor ... t_null_type<c10::StorageImpl> >&&, c10::DispatchKey&&, caffe2::TypeMeta&)
+                2000  /usr/include/c++/8/array:at::Tensor& c10::Dispatcher::callWithDispatchKe ... , c10::Scalar)> const&, c10::DispatchKey, at::Tensor&, c10::Scalar) const
 
-            Total: 8863284"""  # noqa
+            Total: 8869966""",  # noqa: B950
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -669,20 +747,20 @@ class TestBenchmarkUtils(TestCase):
               192000  build/../c10/core/Device.h:c10::Device::validate()
               169855  ???:_int_malloc
               154000  build/../c10/core/TensorOptions. ... erge_in(c10::TensorOptions) const
-              147167  Python/ceval.c:_PyEval_EvalFrameDefault
+              148561  Python/ceval.c:_PyEval_EvalFrameDefault
               135000  ???:malloc
                  ...
-                 -62  Objects/abstract.c:PyNumber_Subtract
-                 -63  Objects/longobject.c:long_richcompare
-                 -70  Objects/abstract.c:_PyObject_FastCallDict
-                 -74  Python/pytime.c:_PyTime_FromSecondsObject
-                 -85  /home/test_user/miniconda3/envs/ ... her(_object*, _object*, _object*)
-                 -95  Objects/rangeobject.c:rangeiter_next
-                -104  Objects/object.c:PyObject_RichCompare
-                -134  Objects/rangeobject.c:range_new
-                -180  Objects/longobject.c:PyLong_FromLong
+                2000  /usr/include/c++/8/ext/new_allocator.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/stl_vect ... *, _object*, _object*, _object**)
+                2000  /usr/include/c++/8/bits/stl_vect ... rningHandler::~PyWarningHandler()
+                2000  /usr/include/c++/8/bits/stl_vect ... ject*, _object*, _object**, bool)
+                2000  /usr/include/c++/8/bits/stl_algobase.h:torch::PythonArgs::intlist(int)
+                2000  /usr/include/c++/8/bits/shared_p ... ad_accumulator(at::Tensor const&)
+                2000  /usr/include/c++/8/bits/move.h:c ... te<c10::AutogradMetaInterface> >)
+                2000  /usr/include/c++/8/bits/atomic_b ... DispatchKey&&, caffe2::TypeMeta&)
+                2000  /usr/include/c++/8/array:at::Ten ... , at::Tensor&, c10::Scalar) const
 
-            Total: 8863284""",
+            Total: 8869966""",
         )
 
         self.regularizeAndAssertExpectedInline(
@@ -773,10 +851,8 @@ class TestBenchmarkUtils(TestCase):
         costs = (
             # overhead_optimized_fn()
             (1e-6, 1e-9),
-
             # compute_optimized_fn()
             (3e-6, 5e-10),
-
             # special_case_fn()  [square inputs only]
             (1e-6, 4e-10),
         )
@@ -792,8 +868,7 @@ class TestBenchmarkUtils(TestCase):
         # overhead_optimized_fn()
         class _MockTimer_0(self._MockTimer):
             _function_costs = tuple(
-                (f"fn({i}, {j})", costs[0][0] + costs[0][1] * i * j)
-                for i, j in sizes
+                (f"fn({i}, {j})", costs[0][0] + costs[0][1] * i * j) for i, j in sizes
             )
 
         class MockTimer_0(benchmark_utils.Timer):
@@ -802,8 +877,7 @@ class TestBenchmarkUtils(TestCase):
         # compute_optimized_fn()
         class _MockTimer_1(self._MockTimer):
             _function_costs = tuple(
-                (f"fn({i}, {j})", costs[1][0] + costs[1][1] * i * j)
-                for i, j in sizes
+                (f"fn({i}, {j})", costs[1][0] + costs[1][1] * i * j) for i, j in sizes
             )
 
         class MockTimer_1(benchmark_utils.Timer):
@@ -813,7 +887,8 @@ class TestBenchmarkUtils(TestCase):
         class _MockTimer_2(self._MockTimer):
             _function_costs = tuple(
                 (f"fn({i}, {j})", costs[2][0] + costs[2][1] * i * j)
-                for i, j in sizes if i == j
+                for i, j in sizes
+                if i == j
             )
 
         class MockTimer_2(benchmark_utils.Timer):
@@ -865,7 +940,7 @@ class TestBenchmarkUtils(TestCase):
                   compute_optimized      |    3.1     |     4.0     |     11.2     |     2099.3     |     2099.3
                   special_case (square)  |    1.1     |             |      7.5     |                |     1674.7
 
-            Times are in microseconds (us)."""
+            Times are in microseconds (us).""",
         )
 
         compare.trim_significant_figures()
@@ -879,7 +954,7 @@ class TestBenchmarkUtils(TestCase):
                   compute_optimized      |     3      |     4.0     |      11      |      2100      |      2100
                   special_case (square)  |     1      |             |       8      |                |      1700
 
-            Times are in microseconds (us)."""
+            Times are in microseconds (us).""",
         )
 
         compare.colorize()
@@ -893,7 +968,7 @@ class TestBenchmarkUtils(TestCase):
                   compute_optimized      |  \x1b[2m\x1b[91m   3    \x1b[0m\x1b[0m  |     4.0     |      11      |  \x1b[92m\x1b[1m    2100    \x1b[0m\x1b[0m  |      2100
                   special_case (square)  |  \x1b[92m\x1b[1m   1    \x1b[0m\x1b[0m  |             |  \x1b[92m\x1b[1m     8    \x1b[0m\x1b[0m  |                |  \x1b[92m\x1b[1m    1700    \x1b[0m\x1b[0m
 
-            Times are in microseconds (us)."""  # noqa
+            Times are in microseconds (us)."""  # noqa: B950
         )
 
         compare.colorize(rowwise=True)
@@ -907,7 +982,7 @@ class TestBenchmarkUtils(TestCase):
                   compute_optimized      |  \x1b[92m\x1b[1m   3    \x1b[0m\x1b[0m  |     4.0     |  \x1b[2m\x1b[91m    11    \x1b[0m\x1b[0m  |  \x1b[31m\x1b[1m    2100    \x1b[0m\x1b[0m  |  \x1b[31m\x1b[1m    2100    \x1b[0m\x1b[0m
                   special_case (square)  |  \x1b[92m\x1b[1m   1    \x1b[0m\x1b[0m  |             |  \x1b[31m\x1b[1m     8    \x1b[0m\x1b[0m  |                |  \x1b[31m\x1b[1m    1700    \x1b[0m\x1b[0m
 
-            Times are in microseconds (us)."""  # noqa
+            Times are in microseconds (us)."""  # noqa: B950
         )
 
         def print_new_expected(s: str) -> None:
@@ -930,12 +1005,16 @@ class TestBenchmarkUtils(TestCase):
         self.assertEqual(columnwise_colored_actual, columnwise_colored_expected)
         self.assertEqual(rowwise_colored_actual, rowwise_colored_expected)
 
-    @unittest.skipIf(IS_WINDOWS and os.getenv("VC_YEAR") == "2019", "Random seed only accepts int32")
+    @unittest.skipIf(
+        IS_WINDOWS and os.getenv("VC_YEAR") == "2019", "Random seed only accepts int32"
+    )
     def test_fuzzer(self):
         fuzzer = benchmark_utils.Fuzzer(
             parameters=[
                 benchmark_utils.FuzzedParameter(
-                    "n", minval=1, maxval=16, distribution="loguniform")],
+                    "n", minval=1, maxval=16, distribution="loguniform"
+                )
+            ],
             tensors=[benchmark_utils.FuzzedTensor("x", size=("n",))],
             seed=0,
         )
@@ -947,9 +1026,8 @@ class TestBenchmarkUtils(TestCase):
 
         for i, (tensors, _, _) in enumerate(fuzzer.take(2)):
             x = tensors["x"]
-            self.assertEqual(
-                x, torch.Tensor(expected_results[i]), rtol=1e-3, atol=1e-3)
+            self.assertEqual(x, torch.tensor(expected_results[i]), rtol=1e-3, atol=1e-3)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     run_tests()
