@@ -1,10 +1,12 @@
+# Owner(s): ["module: unknown"]
+
 import torch
 import torch.nn as nn
 
 
 class Model(nn.Module):
-    def __init__(self):
-        super(Model, self).__init__()
+    def __init__(self) -> None:
+        super().__init__()
         self.linear = nn.Linear(20, 20)
 
     def forward(self, input):
@@ -16,12 +18,12 @@ def main():
     data = torch.randn(10, 50).cuda()
     model = Model().cuda()
     optimizer = torch.optim.SGD(model.parameters(), lr=0.0001)
-    for i in range(10):
+    for _ in range(10):
         optimizer.zero_grad()
         loss = model(data)
         loss.backward()
         optimizer.step()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
