@@ -2,7 +2,6 @@
 
 import contextlib
 import sys
-import unittest
 
 import torch
 import torch._dynamo.config
@@ -20,7 +19,16 @@ from torch.testing._internal.common_utils import (
 
 
 class CustomException(Exception):
-    ...
+    pass
+
+
+class CustomExceptionMeta(type):
+    def __instancecheck__(cls, instance):
+        return True
+
+
+class CustomExceptionWithInstanceCheck(Exception, metaclass=CustomExceptionMeta):
+    pass
 
 
 class CustomExceptionWithArgs(Exception):
@@ -150,6 +158,14 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         out = f(inp)
         self.assertTrue(torch.equal(out, inp + 1))
 
+    @make_dynamo_test
+    def test_isinstance_CustomException(self):
+        assert isinstance(CustomException, type)
+        assert not isinstance(CustomException(), type)
+        C = CustomExceptionWithInstanceCheck
+        assert isinstance(C, C)
+        assert isinstance(C(), C)
+
     @make_dynamo_test
     def test_propagate_exception_inside_ctx_manager(self):
         @contextlib.contextmanager
@@ -199,7 +215,6 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         res = opt_fn(x)
         self.assertEqual(ref, res)
 
-    @unittest.skipIf(sys.version_info < (3, 11), "Python 3.11+")
     @make_dynamo_test
     def test_raise_match(self):
         a = AttributeError
@@ -281,7 +296,6 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         opt_fn = torch.compile(fn, backend="eager")
         opt_fn(x)
 
-    @unittest.skipIf(sys.version_info < (3, 11), "Python 3.11+")
     def test_exception_with_ctx_manager(self):
         def fn(x):
             x = torch.cos(x)
@@ -344,7 +358,7 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
 
     def test_raise_custom_exception(self):
         class Exc(Exception):
-            ...
+            pass
 
         @torch.compile(backend="eager", fullgraph=True)
         def fn(t):
@@ -361,7 +375,7 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
 
     def test_raise_custom_exception_with_args(self):
         class Exc(Exception):
-            ...
+            pass
 
         @torch.compile(backend="eager", fullgraph=True)
         def fn(t):
@@ -567,6 +581,34 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         self.assertEqual(ref[0], res[0])
         self.assertEqual(ref[1], res[1])
 
+    @make_dynamo_test
+    def test_raise_from_None_2(self):
+        def fn():
+            try:
+                raise ValueError
+            except Exception:
+                raise TypeError from None
+
+        try:
+            fn()
+        except TypeError as e:
+            assert e.__cause__ is None
+            assert e.__suppress_context__ is True
+
+    @make_dynamo_test
+    def test_raise_from_other(self):
+        def fn():
+            try:
+                raise ValueError
+            except Exception as e:
+                raise TypeError from e
+
+        try:
+            fn()
+        except TypeError as e:
+            assert isinstance(e.__cause__, ValueError)
+            assert e.__suppress_context__ is True
+
     @make_dynamo_test
     def test_reraise_first_exc(self):
         def fn():
@@ -847,7 +889,6 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         t = torch.randn(2)
         fn(t)
 
-    @unittest.skipIf(sys.version_info < (3, 11), "Python 3.11+")
     def test_user_defined_exception_with_args(self):
         @torch.compile(backend="eager", fullgraph=True)
         def fn(t):
@@ -880,235 +921,6 @@ class ExceptionTests(torch._dynamo.test_case.TestCase):
         assert exc2.__context__ is None
 
 
-class CPythonExceptionTests(torch._dynamo.test_case.TestCase):
-    # Tests taken from CPython source code in cpython/Lib/test/test_exceptions.py
-    # https://github.com/python/cpython/blob/v3.13.1/Lib/test/test_exceptions.py
-
-    @make_dynamo_test
-    def testChainingAttrs(self):
-        e = Exception()
-        assert e.__context__ is None
-        assert e.__cause__ is None
-
-        e = TypeError()
-        assert e.__context__ is None
-        assert e.__cause__ is None
-
-        e = MyException()
-        assert e.__context__ is None
-        assert e.__cause__ is None
-
-    @make_dynamo_test
-    def testChainingDescriptors(self):
-        try:
-            raise Exception  # noqa: TRY002
-        except Exception as exc:
-            e = exc
-
-        assert e.__context__ is None
-        assert e.__cause__ is None
-        assert e.__suppress_context__ is False
-
-        e.__context__ = NameError()
-        e.__cause__ = None
-        assert isinstance(e.__context__, NameError)
-        assert e.__cause__ is None
-        assert e.__suppress_context__ is True
-        e.__suppress_context__ = False
-        assert e.__suppress_context__ is False
-
-    @make_dynamo_test
-    def test_context_of_exception_in_try_and_finally(self):
-        try:
-            try:
-                te = TypeError(1)
-                raise te
-            finally:
-                ve = ValueError(2)
-                raise ve
-        except Exception as e:
-            exc = e
-
-        assert exc is ve
-        assert exc.__context__ is te
-
-    @make_dynamo_test
-    def test_context_of_exception_in_except_and_finally(self):
-        try:
-            try:
-                te = TypeError(1)
-                raise te
-            except Exception:  # noqa: E722
-                ve = ValueError(2)
-                raise ve  # noqa: B904
-            finally:
-                oe = OSError(3)
-                raise oe
-        except Exception as e:
-            exc = e
-
-        assert exc is oe
-        assert exc.__context__ is ve
-        assert exc.__context__.__context__ is te
-
-    @make_dynamo_test
-    def test_context_of_exception_in_else_and_finally(self):
-        try:
-            try:
-                pass
-            except Exception:  # noqa: E722
-                pass
-            else:
-                ve = ValueError(1)
-                raise ve
-            finally:
-                oe = OSError(2)
-                raise oe
-        except Exception as e:
-            exc = e
-
-        assert exc is oe
-        assert exc.__context__ is ve
-
-    @unittest.expectedFailure
-    @make_dynamo_test
-    def test_raise_does_not_create_context_chain_cycle(self):
-        A = AssertionError
-        B = BytesWarning
-        C = ConnectionError
-
-        # Create a context chain:
-        # C -> B -> A
-        # Then raise A in context of C.
-        try:
-            try:
-                raise A
-            except A as a_:
-                a = a_
-                try:
-                    raise B
-                except B as b_:
-                    b = b_
-                    try:
-                        raise C
-                    except C as c_:
-                        c = c_
-                        self.assertIsInstance(a, A)
-                        self.assertIsInstance(b, B)
-                        self.assertIsInstance(c, C)
-                        self.assertIsNone(a.__context__)
-                        self.assertIs(b.__context__, a)
-                        self.assertIs(c.__context__, b)
-                        raise a  # noqa: B904
-        except A as e:
-            exc = e
-
-        # Expect A -> C -> B, without cycle
-        self.assertIs(exc, a)
-        self.assertIs(a.__context__, c)
-        self.assertIs(c.__context__, b)
-        self.assertIsNone(b.__context__)
-
-    @unittest.expectedFailure
-    @make_dynamo_test
-    def test_no_hang_on_context_chain_cycle1(self):
-        # See issue 25782. Cycle in context chain.
-
-        def cycle():
-            try:
-                raise ValueError(1)
-            except ValueError as ex:
-                ex.__context__ = ex
-                raise TypeError(2)  # noqa: B904
-
-        try:
-            cycle()
-        except Exception as e:
-            exc = e
-
-        self.assertIsInstance(exc, TypeError)
-        self.assertIsInstance(exc.__context__, ValueError)
-        self.assertIs(exc.__context__.__context__, exc.__context__)
-
-    @unittest.expectedFailure
-    @make_dynamo_test
-    def test_no_hang_on_context_chain_cycle2(self):
-        # See issue 25782. Cycle at head of context chain.
-
-        A = AssertionError
-        B = BytesWarning
-        C = ConnectionError
-
-        # Context cycle:
-        # +-----------+
-        # V           |
-        # C --> B --> A
-        with self.assertRaises(C) as cm:
-            try:
-                raise A()  # noqa: RSE102
-            except A as _a:
-                a = _a
-                try:
-                    raise B()  # noqa: RSE102
-                except B as _b:
-                    b = _b
-                    try:
-                        raise C()  # noqa: RSE102
-                    except C as _c:
-                        c = _c
-                        a.__context__ = c
-                        raise c  # noqa: B904
-
-        self.assertIs(cm.exception, c)
-        # Verify the expected context chain cycle
-        self.assertIs(c.__context__, b)
-        self.assertIs(b.__context__, a)
-        self.assertIs(a.__context__, c)
-
-    @unittest.expectedFailure
-    @make_dynamo_test
-    def test_no_hang_on_context_chain_cycle3(self):
-        # See issue 25782. Longer context chain with cycle.
-        A = AssertionError
-        B = BytesWarning
-        C = ConnectionError
-        D = DeprecationWarning
-        E = Exception
-
-        # Context cycle:
-        #             +-----------+
-        #             V           |
-        # E --> D --> C --> B --> A
-        with self.assertRaises(E) as cm:
-            try:
-                raise A
-            except A as _a:
-                a = _a
-                try:
-                    raise B
-                except B as _b:
-                    b = _b
-                    try:
-                        raise C
-                    except C as _c:
-                        c = _c
-                        a.__context__ = c
-                        try:
-                            raise D
-                        except D as _d:
-                            d = _d
-                            e = E()
-                            raise e  # noqa: B904
-
-        self.assertIs(cm.exception, e)
-        # Verify the expected context chain cycle
-        self.assertIs(e.__context__, d)
-        self.assertIs(d.__context__, c)
-        self.assertIs(c.__context__, b)
-        self.assertIs(b.__context__, a)
-        self.assertIs(a.__context__, c)
-
-
 instantiate_parametrized_tests(ExceptionTests)
 
 
